package cn.com.ctop.kuaishou.modules.report.service.impl;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.HttpUtils;
import cn.com.ctop.common.module.utils.KuaishouInterfaceConstant;
import cn.com.ctop.common.module.utils.PropertiesUtils;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDaily;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDailyAge;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDailyBussiness;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDailyCity;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDailyClient;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDailyGender;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDailyProvince;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyAgeMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyBusinessMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyCityMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyClientMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyGenderMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyProvinceMapper;
import cn.com.ctop.kuaishou.modules.report.service.IKuaishouAudienceReportDailyService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * 人群分析报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-12-03
 */
@Slf4j
@Service
public class KuaishouAudienceReportDailyServiceImpl extends ServiceImpl<KuaishouAudienceReportDailyMapper, KuaishouAudienceReportDaily> implements IKuaishouAudienceReportDailyService {

    @Autowired
    private KuaishouAudienceReportDailyProvinceMapper provinceMapper;
    @Autowired
    private KuaishouAudienceReportDailyAgeMapper ageMapper;
    @Autowired
    private KuaishouAudienceReportDailyCityMapper cityMapper;
    @Autowired
    private KuaishouAudienceReportDailyClientMapper clientMapper;
    @Autowired
    private KuaishouAudienceReportDailyBusinessMapper bussinessMapper;
    @Autowired
    private KuaishouAudienceReportDailyGenderMapper genderMapper;
    @Autowired
    private ICtopOauthTokenService tokenService;
    private String urlPath = PropertiesUtils.getConfig("kuaishou_api_url");

    /**
     * 人群分析报表
     *
     * @param type      province: 省级地域分布, city: 地级地域分布,gender: 性别分布,ageSegment: 年龄分布clientId: 系统分布 businessInterestTags: 商业兴趣分布
     * @param accountId 广告主ID
     * @param file      请求的文件，最大支持500MB
     * @return
     */
    @Override
    public void crowdAnalysisReport(CtopOauthToken token, String date, String type, Long plan, Long unit, Integer page, Integer pageSize) {
        long start = System.currentTimeMillis();
        try {
            String url = urlPath + KuaishouInterfaceConstant.AUDIENCE_REPORT;
            JSONArray reportDims = new JSONArray();
            reportDims.add(type);
            Map<String, String> headers = new HashMap<>();
            headers.put("Access-Token", token.getAccessToken());
            headers.put("Content-Type", "application/json");
            JSONObject requestJson = new JSONObject();
            requestJson.put("advertiser_id", token.getAccountId());
            requestJson.put("start_date", date);
            requestJson.put("end_date", date);
            requestJson.put("report_dims", reportDims);
            requestJson.put("page", page);
            requestJson.put("page_size", pageSize);
            if (!Check.isNull(plan)) {
                JSONArray plans = new JSONArray();
                plans.add(plan);
                requestJson.put("campaign_ids", plans);
            }
            if (!Check.isNull(unit)) {
                JSONArray units = new JSONArray();
                units.add(unit);
                requestJson.put("unit_ids", units);
            }
            String result = HttpUtils.kuaiShouhttpPostRequest(url, requestJson.toJSONString(), headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (!Check.isNull(resultJson)) {
                Integer code = resultJson.getInteger("code");
                if (code == 0) {
                    JSONObject datas = resultJson.getJSONObject("data");
                    JSONArray details = datas.getJSONArray("details");
                    if (!Check.isNull(details)) {
                        for (int i = 0; i < details.size(); i++) {
                            JSONObject obj = details.getJSONObject(i);
                            if ("province".equals(type)) {
                                KuaishouAudienceReportDailyProvince entity = JSONObject.toJavaObject(obj, KuaishouAudienceReportDailyProvince.class);
                                entity.setPhotoLike(obj.getLong("like"));
                                entity.setComments(obj.getLong("comment"));
                                entity.setPhotoShow(obj.getLong("show"));
                                entity.setShares(obj.getString("share"));
                                entity.setAccountId(Long.valueOf(token.getAccountId()));
                                entity.setPlanId(plan);
                                entity.setUnitId(unit);
                                provinceMapper.insert(entity);
                            } else if ("city".equals(type)) {
                                KuaishouAudienceReportDailyCity entity = JSONObject.toJavaObject(obj, KuaishouAudienceReportDailyCity.class);
                                entity.setPhotoLike(obj.getLong("like"));
                                entity.setComments(obj.getLong("comment"));
                                entity.setPhotoShow(obj.getLong("show"));
                                entity.setShares(obj.getString("share"));
                                entity.setAccountId(Long.valueOf(token.getAccountId()));
                                entity.setPlanId(plan);
                                entity.setUnitId(unit);
                                cityMapper.insert(entity);
                            } else if ("gender".equals(type)) {
                                KuaishouAudienceReportDailyGender entity = JSONObject.toJavaObject(obj, KuaishouAudienceReportDailyGender.class);
                                entity.setPhotoLike(obj.getLong("like"));
                                entity.setComments(obj.getLong("comment"));
                                entity.setPhotoShow(obj.getLong("show"));
                                entity.setShares(obj.getString("share"));
                                entity.setAccountId(Long.valueOf(token.getAccountId()));
                                entity.setPlanId(plan);
                                entity.setUnitId(unit);
                                genderMapper.insert(entity);
                            } else if ("ageSegment".equals(type)) {
                                KuaishouAudienceReportDailyAge entity = JSONObject.toJavaObject(obj, KuaishouAudienceReportDailyAge.class);
                                entity.setPhotoLike(obj.getLong("like"));
                                entity.setComments(obj.getLong("comment"));
                                entity.setPhotoShow(obj.getLong("show"));
                                entity.setShares(obj.getString("share"));
                                entity.setAccountId(Long.valueOf(token.getAccountId()));
                                entity.setPlanId(plan);
                                entity.setUnitId(unit);
                                ageMapper.insert(entity);
                            } else if ("clientId".equals(type)) {
                                KuaishouAudienceReportDailyClient entity = JSONObject.toJavaObject(obj, KuaishouAudienceReportDailyClient.class);
                                entity.setPhotoLike(obj.getLong("like"));
                                entity.setComments(obj.getLong("comment"));
                                entity.setPhotoShow(obj.getLong("show"));
                                entity.setShares(obj.getString("share"));
                                entity.setAccountId(Long.valueOf(token.getAccountId()));
                                entity.setPlanId(plan);
                                entity.setUnitId(unit);
                                clientMapper.insert(entity);
                            } else if ("businessInterestTags".equals(type)) {
                                KuaishouAudienceReportDailyBussiness entity = JSONObject.toJavaObject(obj, KuaishouAudienceReportDailyBussiness.class);
                                entity.setPhotoLike(obj.getLong("like"));
                                entity.setComments(obj.getLong("comment"));
                                entity.setPhotoShow(obj.getLong("show"));
                                entity.setShares(obj.getString("share"));
                                entity.setAccountId(Long.valueOf(token.getAccountId()));
                                entity.setPlanId(plan);
                                entity.setUnitId(unit);
                                bussinessMapper.insert(entity);
                            }
                        }
                    }
                } else {
                    log.error("查询人群分析报表数据返回信息错误：accountId：{}，返回信息：{}", token.getAccountId(), resultJson);
                }
            }
        } catch (Exception e) {
            log.error("插入人群分析报表数据异常，数据维度：{}", type, e);
        }
        log.info("use time ：{} ms", (System.currentTimeMillis() - start));
    }
}
